@@ -23,6 +23,7 @@
 #include "sebie_view.h"
 #include "sebie_gtk_helpers.h"
 #include "sebie_config.h"
+#include "sebie_callbacks.h"
 #include "config.h"
 
 #include <gdk/gdkkeysyms.h>
@@ -30,9 +31,17 @@
 #include <stdio.h>
 #include <string.h>
 
-
+#if GTK_CHECK_VERSION(4,0,0)
+void
+sebie_update_view(struct sebie_s *sebie,
+                  GtkDrawingArea *area,
+                  cairo_t *cr,
+                  int width,
+                  int height)
+#else
 void
 sebie_update_view(struct sebie_s *sebie)
+#endif
 {
   if (sebie->dirty_tags & SEBIE_UPDATE_SELECTION_FAC) {
     gchar buf[6];
@@ -44,26 +53,49 @@ sebie_update_view(struct sebie_s *sebie)
   if (sebie->dirty_tags & SEBIE_UPDATE_SELECTION_FAC_W) {
     gchar buf[6];
     snprintf(buf, 6, "%d", sebie->model.selection_fac_width);
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(sebie->view.entry_selection_fac_width));
+    gtk_entry_buffer_set_text(entry_buffer, buf, -1);
+#else
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_selection_fac_width), buf);
+#endif
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_SELECTION_FAC_H) {
     gchar buf[6];
     snprintf(buf, 6, "%d", sebie->model.selection_fac_height);
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(sebie->view.entry_selection_fac_height));
+    gtk_entry_buffer_set_text(entry_buffer, buf, -1);
+#else
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_selection_fac_height), buf);
+#endif
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_IMAGE_GAMMA) {
+#if GTK_CHECK_VERSION(4,0,0)
+    gtk_spin_button_set_value (GTK_SPIN_BUTTON(sebie->view.entry_selection_gamma), sebie->model.image_gamma);
+#else
     gchar buf[6];
     snprintf(buf, 6, "%1.1f", sebie->model.image_gamma);
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_selection_gamma), buf);
+#endif
   }
 
+#if GTK_CHECK_VERSION(4,0,0)
   if (sebie->dirty_tags & SEBIE_UPDATE_SCALED_PIXBUF ||
       sebie->dirty_tags & SEBIE_UPDATE_SELECTION_RECT) {
+      gtk_widget_queue_draw(sebie->view.drawingarea1);
+  }
+#else
+  if (sebie->dirty_tags & SEBIE_UPDATE_SCALED_PIXBUF ||
+      sebie->dirty_tags & SEBIE_UPDATE_SELECTION_RECT) {
+#endif
     /* do it the rough way, always draw the whole area and not just the
      * areas which changed  */
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  if (area != NULL && cr != NULL) {
+#elif GTK_CHECK_VERSION(3,0,0)
 #if GTK_CHECK_VERSION(3,22,0)
     GdkWindow *window = gtk_widget_get_window(sebie->view.drawingarea1);
     cairo_region_t *cairo_region = cairo_region_create();
@@ -151,7 +183,8 @@ sebie_update_view(struct sebie_s *sebie)
                         rheight);
 #endif
 
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+#elif GTK_CHECK_VERSION(3,0,0)
 #if GTK_CHECK_VERSION(3,22,0)
     gdk_window_end_draw_frame(window,drawing_context);
     cairo_region_destroy(cairo_region);
@@ -163,21 +196,41 @@ sebie_update_view(struct sebie_s *sebie)
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_INPUT_PATH) {
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(sebie->view.entry_input_path));
+    gtk_entry_buffer_set_text(entry_buffer, sebie->model.input_path, -1);
+#else
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_input_path),
                        sebie->model.input_path);
+#endif
   }
   if (sebie->dirty_tags & SEBIE_UPDATE_INPUT_REGEX) {
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(sebie->view.entry_input_regex));
+    gtk_entry_buffer_set_text(entry_buffer, sebie->model.input_regex, -1);
+#else
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_input_regex),
                        sebie->model.input_regex);
+#endif
   }
   if (sebie->dirty_tags & SEBIE_UPDATE_OUTPUT_FORMAT) {
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(sebie->view.entry_output_format));
+    gtk_entry_buffer_set_text(entry_buffer, sebie->model.output_format, -1);
+#else
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_output_format),
                        sebie->model.output_format);
+#endif
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_OUTPUT_FILE_NAME) {
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(sebie->view.entry_output_file));
+    gtk_entry_buffer_set_text(entry_buffer, sebie->model.output_file_name, -1);
+#else
     gtk_entry_set_text(GTK_ENTRY(sebie->view.entry_output_file),
                        sebie->model.output_file_name);
+#endif
   }
 
   if (sebie->dirty_tags & SEBIE_UPDATE_STATUS_BAR) {
@@ -199,16 +252,17 @@ sebie_error_popup(struct sebie_s *sebie, const gchar *error_str)
                                               GTK_BUTTONS_CLOSE,
                                               "%s",
                                               error_str);
-#if 1
+
   // modal dialog: you must click to resume
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
+  g_signal_connect (G_OBJECT (dialog), "response",
+                    G_CALLBACK (on_error_dialog_response),
+                    NULL);
+  gtk_widget_show (dialog);
+#else
   gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
-#else
-  // non modal dialog: application will run further
-  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
-                            G_CALLBACK (gtk_widget_destroy),
-                            GTK_OBJECT (dialog));
-  gtk_widget_show (dialog);
 #endif
 }
 
@@ -303,11 +357,10 @@ load_pixbuf (struct sebie_s *sebie)
 void
 sebie_update_file_list(struct sebie_s *sebie)
 {
-  GError *error = NULL;
   GDir *gdir;
   save_sebie_config(sebie);
   sebie_file_list_clear(sebie);
-  if ((gdir = g_dir_open(sebie->model.input_path, 0, &error)) == NULL) {
+  if ((gdir = g_dir_open(sebie->model.input_path, 0, NULL)) == NULL) {
     return;
   }
 
@@ -323,6 +376,8 @@ sebie_update_file_list(struct sebie_s *sebie)
       sebie_debug("sebie->model.regex_status invalid");
     }
   }
+
+  g_dir_close(gdir);
 }
 
 
@@ -342,7 +397,11 @@ rotate_scaled(struct sebie_s *sebie, int direction)
 
     sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
+#if GTK_CHECK_VERSION(4,0,0)
+    sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
     sebie_update_view(sebie);
+#endif
   }
 }
 
@@ -370,7 +429,11 @@ apply_color_corrections(struct sebie_s *sebie)
     sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
     sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
   }
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
   sebie_update_view(sebie);
+#endif
 }
 
 
@@ -429,10 +492,13 @@ init_sebie_model(struct sebie_s *sebie)
   };
 
   sebie_read_config(sebie->config_file, config_spec);
+  if (sebie->model.selection_fac_width < 1) {
 
-  sebie->model.input_file_path = NULL;
-  sebie->model.output_file_path = NULL;
-
+      sebie->model.selection_fac_width = 1;
+  }
+  if (sebie->model.selection_fac_height < 1) {
+      sebie->model.selection_fac_height = 1;
+  }
   sebie->model.regex_status = 0;
 
   sebie->dirty_tags |= SEBIE_UPDATE_INPUT_PATH;
@@ -450,12 +516,52 @@ init_sebie_model(struct sebie_s *sebie)
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_H;
   sebie->dirty_tags |= SEBIE_UPDATE_IMAGE_GAMMA;
 
-  sebie->model.statusbar_text = strdup("   SeBIE-" PACKAGE_VERSION " (C) 2002-2010 by Peter.Seiderer@ciselant.de");
+  sebie->model.statusbar_text = strdup("   SeBIE-" PACKAGE_VERSION " (c) 2002-2022 by Peter.Seiderer@ciselant.de");
   sebie->dirty_tags |= SEBIE_UPDATE_STATUS_BAR;
 
   sebie_regcomp(sebie);
 }
 
+void
+deinit_sebie_model(struct sebie_s *sebie)
+{
+    if (sebie->model.input_path != NULL) {
+        g_free(sebie->model.input_path);
+    }
+    if (sebie->model.input_regex != NULL) {
+        g_free(sebie->model.input_regex);
+    }
+    if (sebie->model.output_format != NULL) {
+        g_free(sebie->model.output_format);
+    }
+    if (sebie->model.input_file_name != NULL) {
+        g_free(sebie->model.input_file_name);
+    }
+    if (sebie->model.output_file_name != NULL) {
+        g_free(sebie->model.output_file_name);
+    }
+
+    if (sebie->model.regex_status) {
+        regfree(&sebie->model.regex);
+        sebie->model.regex_status = 0;
+    }
+
+    if (sebie->model.image_pixbuf != NULL) {
+        g_object_unref(sebie->model.image_pixbuf);
+    }
+
+    if (sebie->model.scaled_pixbuf_orig != NULL) {
+        g_object_unref(sebie->model.scaled_pixbuf_orig);
+    }
+
+    if (sebie->model.scaled_pixbuf != NULL) {
+        g_object_unref(sebie->model.scaled_pixbuf);
+    }
+
+    g_free(sebie->model.statusbar_text);
+
+    g_free(sebie->config_file);
+}
 
 int main(int argc, char *argv[])
 {
@@ -469,15 +575,33 @@ int main(int argc, char *argv[])
   gtk_set_locale ();
 #endif
 
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_init ();
+#else
   gtk_init (&argc, &argv);
+#endif
 
   init_sebie_model(sebie);
   window1 = init_sebie_view(sebie);
+
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
   sebie_update_view(sebie);
+#endif
 
   gtk_widget_show (window1);
 
+#if GTK_CHECK_VERSION(4,0,0)
+  while (g_list_model_get_n_items (gtk_window_get_toplevels ()) > 0) {
+    g_main_context_iteration (NULL, TRUE);
+  }
+#else
   gtk_main ();
+#endif
+
+  deinit_sebie_model(sebie);
+  g_free(sebie);
 
   exit(0);
 }
