@@ -74,18 +74,24 @@ typedef enum sebie_resize_corner_e {
 
 typedef struct sebie_view_s {
   GtkWidget *window1;
-  GtkWidget *vbox1;
-  GtkWidget *toolbar1;
-  GtkWidget *tmp_toolbar_icon;
 #if GTK_CHECK_VERSION(3,0,0)
+  GtkWidget *grid1;
+#else
+  GtkWidget *vbox1;
+#endif
+  GtkWidget *toolbar1;
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkWidget *button_quit;
+  GtkWidget *button_save;
+  GtkWidget *button_rotate_left;
+  GtkWidget *button_rotate_right;
+#elif GTK_CHECK_VERSION(3,0,0)
   GtkToolItem *button_quit;
-  //GtkWidget *button_load;
   GtkToolItem *button_save;
   GtkToolItem *button_rotate_left;
   GtkToolItem *button_rotate_right;
 #else
   GtkWidget *button_quit;
-  //GtkWidget *button_load;
   GtkWidget *button_save;
   GtkWidget *button_rotate_left;
   GtkWidget *button_rotate_right;
@@ -94,7 +100,9 @@ typedef struct sebie_view_s {
   GtkWidget *entry_selection_fac_width;
   GtkWidget *label_selection_fac_height;
   GtkWidget *entry_selection_fac_height;
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkWidget *button_fac_swap;
+#elif GTK_CHECK_VERSION(3,0,0)
   GtkToolItem *button_fac_swap;
 #else
   GtkWidget *button_fac_swap;
@@ -102,12 +110,16 @@ typedef struct sebie_view_s {
   GtkWidget *label_selection_fac;
   GtkWidget *label_selection_gamma;
   GtkWidget *entry_selection_gamma;
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkWidget *button_grayscale;
+#elif GTK_CHECK_VERSION(3,0,0)
   GtkToolItem *button_grayscale;
 #else
   GtkWidget *button_grayscale;
 #endif
+#if !GTK_CHECK_VERSION(3,0,0)
   GtkWidget *hpaned1;
+#endif
   GtkWidget *vbox2;
   GtkWidget *label_input_path;
   GtkWidget *entry_input_path;
@@ -131,13 +143,10 @@ typedef struct sebie_model_s {
   char *output_format;
   char *input_file_name;
   char *output_file_name;
-  char *input_file_path;
-  char *output_file_path;
   regex_t regex;
   int regex_status;
   GtkListStore *file_list;
   /* orig image */
-  char *image_file_name;
   GdkPixbuf *image_pixbuf;
   gint image_width;
   gint image_height;
@@ -182,8 +191,15 @@ typedef struct sebie_s {
   char *config_file;
 } struct_sebie;
 
-
+#if GTK_CHECK_VERSION(4,0,0)
+void sebie_update_view(struct sebie_s *sebie,
+                       GtkDrawingArea *area,
+                       cairo_t *cr,
+                       int width,
+                       int height);
+#else
 void sebie_update_view(struct sebie_s *sebie);
+#endif
 void sebie_error_popup(struct sebie_s *sebie, const gchar *error_str);
 void load_pixbuf (struct sebie_s *sebie);
 void sebie_update_file_list(struct sebie_s *sebie);
