@@ -40,6 +40,7 @@ on_treeview_file_list_changed(GtkTreeSelection *selection,
     if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
         gtk_tree_model_get(model, &iter, 0, &filename, -1);
         sebie_set_string(&sebie->model.input_file_name, filename);
+        g_free(filename);
         load_pixbuf(sebie);
         if (sebie->model.regex_status) {
             int nmatch = 2;
@@ -56,7 +57,11 @@ on_treeview_file_list_changed(GtkTreeSelection *selection,
               snprintf(buf, 256, sebie->model.output_format, buf_regex[1]);
               sebie_set_string(&sebie->model.output_file_name, buf);
               sebie->dirty_tags |= SEBIE_UPDATE_OUTPUT_FILE_NAME;
+#if GTK_CHECK_VERSION(4,0,0)
+              sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
               sebie_update_view(sebie);
+#endif
           }
       } else {
           sebie_debug("sebie->model.regex_status invalid");
@@ -69,7 +74,12 @@ on_entry_input_path_changed (GtkEditable *editable,
                  gpointer    user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(editable));
+  const gchar *input_path = gtk_entry_buffer_get_text(entry_buffer);
+#else
   const gchar *input_path = gtk_entry_get_text(GTK_ENTRY(editable));
+#endif
   sebie_set_string(&sebie->model.input_path, input_path);
   sebie->model_tags |= SEBIE_MODEL_INPUT_PATH;
   sebie_update_file_list(sebie);
@@ -81,7 +91,12 @@ on_entry_input_regex_changed (GtkEditable *editable,
                  gpointer    user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(editable));
+  const gchar *input_regex = gtk_entry_buffer_get_text(entry_buffer);
+#else
   const gchar *input_regex = gtk_entry_get_text(GTK_ENTRY(editable));
+#endif
   sebie_debug("input_regex = %s\n", input_regex);
   sebie_set_string(&sebie->model.input_regex, input_regex);
   sebie->model_tags |= SEBIE_MODEL_INPUT_REGEX;
@@ -94,7 +109,12 @@ on_entry_output_format_changed (GtkEditable *editable,
                 gpointer    user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(editable));
+  const gchar *output_format = gtk_entry_buffer_get_text(entry_buffer);
+#else
   const gchar *output_format = gtk_entry_get_text(GTK_ENTRY(editable));
+#endif
   sebie_debug("output_format = %s\n", output_format);
   sebie_set_string(&sebie->model.output_format, output_format);
   sebie->model_tags |= SEBIE_MODEL_OUTPUT_FORMAT;
@@ -117,10 +137,36 @@ window1_destroy (GtkWidget *widget,
          gpointer   data)
 {
     sebie_debug("window1_destroy\n");
+
+#if !GTK_CHECK_VERSION(4,0,0)
     gtk_main_quit ();
+#endif
 }
 
 
+#if GTK_CHECK_VERSION(4,0,0)
+void
+drawingarea1_resize (GtkWidget *widget,
+                     gint width,
+                     gint height,
+                     gpointer user_data)
+{
+    struct sebie_s *sebie = (struct sebie_s *)user_data;
+    gint window1_width;
+    gint window1_height;
+
+    sebie_debug("drawingarea1_resize width = %d height = %d\n", width, height);
+
+    gtk_window_get_default_size(GTK_WINDOW(sebie->view.window1), &window1_width, &window1_height);
+    sebie->model.window1_width = window1_width;
+    sebie->model.window1_height = window1_height;
+    sebie->model.pixbuf_width = width;
+    sebie->model.pixbuf_height = height;
+
+    sebie->model_tags |= SEBIE_MODEL_CHANGE_PIXBUF_SIZE;
+    save_sebie_config(sebie);
+}
+#else
 void
 drawingarea1_size_allocate (GtkWidget *widget,
                 GtkAllocation *allocation,
@@ -145,13 +191,23 @@ drawingarea1_size_allocate (GtkWidget *widget,
     sebie->model_tags |= SEBIE_MODEL_CHANGE_PIXBUF_SIZE;
     save_sebie_config(sebie);
 }
+#endif
 
 
 void
 on_button_quit_clicked (GtkButton *button,
             gpointer  user_data)
 {
+#if GTK_CHECK_VERSION(4,0,0)
+  struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+  sebie_debug("on_button_quit_clicked\n");
+
+  gtk_window_destroy(GTK_WINDOW(sebie->view.window1));
+#else
+  sebie_debug("on_button_quit_clicked\n");
   gtk_main_quit ();
+#endif
 }
 
 
@@ -301,12 +357,31 @@ on_entry_selection_fac_width_changed (GtkEditable *editable,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   if (sebie->dirty_tags == 0) {
-    const gchar *fac_width = gtk_entry_get_text(GTK_ENTRY(editable));
-    sebie_debug("fac_width = %s %d\n", fac_width, atoi(fac_width));
-    sebie->model.selection_fac_width = atoi(fac_width);
-    sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
-    sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(editable));
+    const gchar *fac_width_str = gtk_entry_buffer_get_text(entry_buffer);
+#else
+    const gchar *fac_width_str = gtk_entry_get_text(GTK_ENTRY(editable));
+#endif
+    gchar *end_ptr;
+    errno = 0;
+    gint fac_width = strtol(fac_width_str, &end_ptr, 10);
+    if (errno != 0 || end_ptr == fac_width_str || *end_ptr != '\0' || fac_width < 1) {
+        sebie_debug("fac_width = %s/%d invalid\n", fac_width_str, fac_width);
+        sebie->model.selection_fac_width = 1;
+        sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
+        sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_W;
+    } else {
+        sebie_debug("fac_width = %s %d\n", fac_width_str, fac_width);
+        sebie->model.selection_fac_width = fac_width;
+        sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
+        sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
+    }
+#if GTK_CHECK_VERSION(4,0,0)
+    sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
     sebie_update_view(sebie);
+#endif
   }
 }
 
@@ -316,12 +391,31 @@ on_entry_selection_fac_height_changed (GtkEditable *editable,
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
   if (sebie->dirty_tags == 0) {
-    const gchar *fac_height = gtk_entry_get_text(GTK_ENTRY(editable));
-    sebie_debug("fac_height = %s %d\n", fac_height, atoi(fac_height));
-    sebie->model.selection_fac_height = atoi(fac_height);
-    sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
-    sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
+#if GTK_CHECK_VERSION(4,0,0)
+    GtkEntryBuffer* entry_buffer = gtk_entry_get_buffer(GTK_ENTRY(editable));
+    const gchar *fac_height_str = gtk_entry_buffer_get_text(entry_buffer);
+#else
+    const gchar *fac_height_str = gtk_entry_get_text(GTK_ENTRY(editable));
+#endif
+    gchar *end_ptr;
+    errno = 0;
+    gint fac_height = strtol(fac_height_str, &end_ptr, 10);
+    if (errno != 0 || end_ptr == fac_height_str || *end_ptr != '\0' || fac_height < 1) {
+        sebie_debug("fac_height = %s/%d invalid\n", fac_height_str, fac_height);
+        sebie->model.selection_fac_height = 1;
+        sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
+        sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_H;
+    } else {
+        sebie_debug("fac_height = %s/%d\n", fac_height_str, fac_height);
+        sebie->model.selection_fac_height = fac_height;
+        sebie->model_tags |= SEBIE_MODEL_UPDATE_SELECTION_FAC;
+        sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
+    }
+#if GTK_CHECK_VERSION(4,0,0)
+    sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
     sebie_update_view(sebie);
+#endif
   }
 }
 
@@ -337,7 +431,11 @@ on_button_fac_swap (GtkButton *button,
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC;
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_W;
   sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_FAC_H;
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie_update_view(sebie, NULL, NULL, 0, 0);
+#else
   sebie_update_view(sebie);
+#endif
 }
 
 void
@@ -345,6 +443,9 @@ on_entry_selection_gamma_changed (GtkEditable *editable,
                   gpointer  user_data)
 {
   struct sebie_s *sebie = (struct sebie_s *)user_data;
+#if GTK_CHECK_VERSION(4,0,0)
+  double new_gamma = gtk_spin_button_get_value(GTK_SPIN_BUTTON(editable));
+#else
   const gchar *gamma_str = gtk_entry_get_text(GTK_ENTRY(editable));
   sebie_debug("image_gamma_str = '%s'\n", gamma_str);
   double new_gamma = 0.0;
@@ -352,6 +453,7 @@ on_entry_selection_gamma_changed (GtkEditable *editable,
     sebie_debug("no valid gamma input\n");
     return;
   }
+#endif
   if (new_gamma < 0.1 || new_gamma > 3.0) {
     sebie_debug("new_gamma %lf out of range\n", new_gamma);
     return;
@@ -363,11 +465,22 @@ on_entry_selection_gamma_changed (GtkEditable *editable,
   sebie_debug("set new image_gamma = %f\n", new_gamma);
   sebie->model.image_gamma = new_gamma;
   sebie->model_tags |= SEBIE_MODEL_UPDATE_GAMMA;
-  sebie->dirty_tags |= SEBIE_UPDATE_IMAGE_GAMMA;
   apply_color_corrections(sebie);
 }
 
-#if GTK_CHECK_VERSION(3,0,0)
+
+#if GTK_CHECK_VERSION(4,0,0)
+void
+on_button_grayscale_clicked(GtkToggleButton *button,
+                            gpointer user_data)
+{
+  struct sebie_s *sebie = (struct sebie_s *)user_data;
+  sebie->model.image_grayscale = gtk_toggle_button_get_active (button);
+  sebie_debug("set new image_grayscale = %d\n", sebie->model.image_grayscale);
+  sebie->model_tags |= SEBIE_MODEL_UPDATE_GRAYSCALE;
+  apply_color_corrections(sebie);
+}
+#elif GTK_CHECK_VERSION(3,0,0)
 void
 on_button_grayscale_clicked(GtkToolItem *button,
                 gpointer  user_data)
@@ -391,6 +504,21 @@ on_button_grayscale_clicked(GtkToggleButton *button,
 }
 #endif
 
+
+#if GTK_CHECK_VERSION(4,0,0)
+void
+drawingarea1_draw_function(GtkDrawingArea *area,
+                           cairo_t *cr,
+                           int width,
+                           int height,
+                           gpointer user_data)
+{
+  struct sebie_s *sebie = (struct sebie_s *)user_data;
+  sebie->dirty_tags |= SEBIE_UPDATE_SCALED_PIXBUF;
+  sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
+  sebie_update_view(sebie, area, cr, width, height);
+}
+#else
 gboolean
 on_drawingarea1_expose_event (GtkWidget       *widget,
                   GdkEventExpose  *event,
@@ -404,16 +532,23 @@ on_drawingarea1_expose_event (GtkWidget       *widget,
   sebie_update_view(sebie);
   return FALSE;
 }
+#endif
+
 
 static gboolean
 in_selected_area(gint x, gint y, struct sebie_s *sebie)
 {
+  sebie_debug("in_selected_area: selection_y: %d selection_y: %d selection_width: %d selection_height: %d x: %d y: %d",
+              sebie->model.selection_x, sebie->model.selection_y, sebie->model.selection_width, sebie->model.selection_height, x, y);
+
   if (x >= sebie->model.selection_x &&
       x <= sebie->model.selection_x + sebie->model.selection_width &&
       y >= sebie->model.selection_y &&
       y <= sebie->model.selection_y + sebie->model.selection_height) {
+    sebie_debug(" TRUE\n");
     return TRUE;
   } else {
+    sebie_debug(" FALSE\n");
     return FALSE;
   }
 }
@@ -421,12 +556,17 @@ in_selected_area(gint x, gint y, struct sebie_s *sebie)
 static gboolean
 in_upper_left_corner(gint x, gint y, struct sebie_s *sebie)
 {
+  sebie_debug("in_upper_left_corner: selection_y: %d selection_y: %d selection_width: %d selection_height: %d x: %d y: %d",
+              sebie->model.selection_x, sebie->model.selection_y, sebie->model.selection_width, sebie->model.selection_height, x, y);
+
   if (x < sebie->model.selection_x &&
       x > sebie->model.selection_x-SEBIE_CATCH_PIXEL_COUNT &&
       y < sebie->model.selection_y &&
       y > sebie->model.selection_y-SEBIE_CATCH_PIXEL_COUNT) {
+    sebie_debug(" TRUE\n");
     return TRUE;
   } else {
+    sebie_debug(" FALSE\n");
     return FALSE;
   }
 }
@@ -434,12 +574,17 @@ in_upper_left_corner(gint x, gint y, struct sebie_s *sebie)
 static gboolean
 in_upper_right_corner(gint x, gint y, struct sebie_s *sebie)
 {
+  sebie_debug("in_upper_right_corner: selection_y: %d selection_y: %d selection_width: %d selection_height: %d x: %d y: %d",
+              sebie->model.selection_x, sebie->model.selection_y, sebie->model.selection_width, sebie->model.selection_height, x, y);
+
   if (x > sebie->model.selection_x+sebie->model.selection_width &&
       x < sebie->model.selection_x+sebie->model.selection_width+SEBIE_CATCH_PIXEL_COUNT &&
       y < sebie->model.selection_y &&
       y > sebie->model.selection_y-SEBIE_CATCH_PIXEL_COUNT) {
+    sebie_debug(" TRUE\n");
     return TRUE;
   } else {
+    sebie_debug(" FALSE\n");
     return FALSE;
   }
 }
@@ -447,12 +592,17 @@ in_upper_right_corner(gint x, gint y, struct sebie_s *sebie)
 static gboolean
 in_lower_right_corner(gint x, gint y, struct sebie_s *sebie)
 {
+  sebie_debug("in_lower_right_corner: selection_y: %d selection_y: %d selection_width: %d selection_height: %d x: %d y: %d",
+              sebie->model.selection_x, sebie->model.selection_y, sebie->model.selection_width, sebie->model.selection_height, x, y);
+
   if (x > sebie->model.selection_x+sebie->model.selection_width &&
       x < sebie->model.selection_x+sebie->model.selection_width+SEBIE_CATCH_PIXEL_COUNT &&
       y > sebie->model.selection_y+sebie->model.selection_height &&
       y < sebie->model.selection_y+sebie->model.selection_height+SEBIE_CATCH_PIXEL_COUNT) {
+    sebie_debug(" TRUE\n");
     return TRUE;
   } else {
+    sebie_debug(" FALSE\n");
     return FALSE;
   }
 }
@@ -460,16 +610,230 @@ in_lower_right_corner(gint x, gint y, struct sebie_s *sebie)
 static gboolean
 in_lower_left_corner(gint x, gint y, struct sebie_s *sebie)
 {
+  sebie_debug("in_lower_left_corner: selection_y: %d selection_y: %d selection_width: %d selection_height: %d x: %d y: %d",
+              sebie->model.selection_x, sebie->model.selection_y, sebie->model.selection_width, sebie->model.selection_height, x, y);
+
   if (x < sebie->model.selection_x &&
       x > sebie->model.selection_x-SEBIE_CATCH_PIXEL_COUNT &&
       y > sebie->model.selection_y+sebie->model.selection_height &&
       y < sebie->model.selection_y+sebie->model.selection_height+SEBIE_CATCH_PIXEL_COUNT) {
+    sebie_debug(" TRUE\n");
     return TRUE;
   } else {
+    sebie_debug(" FALSE\n");
     return FALSE;
   }
 }
 
+#if GTK_CHECK_VERSION(4,0,0)
+void
+on_drawingarea1_drag_begin(GtkGestureDrag *gesture,
+                           double x,
+                           double y,
+                           gpointer user_data)
+{
+    struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+    sebie_debug("on_drawingarea1_drag_begin: x: %f y: %f\n", x, y);
+
+    if (in_upper_left_corner(x, y, sebie)) {
+        sebie_debug("UPPER_LEFT\n");
+        sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+        sebie->model.resize_corner = SEBIE_UPPER_LEFT_CORNER;
+    } else if (in_upper_right_corner(x, y, sebie)) {
+        sebie_debug("UPPER_RIGHT\n");
+        sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+        sebie->model.resize_corner = SEBIE_UPPER_RIGHT_CORNER;
+    } else if (in_lower_right_corner(x, y, sebie)) {
+        sebie_debug("LOWER_RIGHT\n");
+        sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+        sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER;
+    } else if (in_lower_left_corner(x, y, sebie)) {
+        sebie_debug("LOWER_LEFT\n");
+        sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+        sebie->model.resize_corner = SEBIE_LOWER_LEFT_CORNER;
+    } else if (in_selected_area(x, y, sebie)) {
+        sebie_debug("MOVE\n");
+        sebie->model.mouse_click_type = SEBIE_MOVE_SELECTION;
+        sebie->model.mouse_pos_lx = x - sebie->model.selection_x;
+        sebie->model.mouse_pos_ly = y - sebie->model.selection_y;
+    } else {
+        sebie_debug("LOWER_RIGHT NEW\n");
+        sebie->model.mouse_click_type = SEBIE_RESIZE_SELECTION;
+        sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER;
+        sebie->model.selection_x = x;
+        sebie->model.selection_y = y;
+        sebie->model.selection_width  = 0;
+        sebie->model.selection_height = 0;
+    }
+
+    sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
+    sebie_update_view(sebie, NULL, NULL, 0, 0);
+}
+
+
+static void
+drawingarea1_update_cursor(gint x, gint y, struct sebie_s* sebie)
+{
+    if (in_selected_area(x,y, sebie) || sebie->model.mouse_click_type == SEBIE_MOVE_SELECTION) {
+        sebie_debug("MOVE\n");
+        gtk_widget_set_cursor_from_name(sebie->view.drawingarea1, "move");
+    } else if (in_upper_left_corner(x, y, sebie) ||
+               (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION && sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER)) {
+        sebie_debug("UPPER_LEFT\n");
+        gtk_widget_set_cursor_from_name(sebie->view.drawingarea1, "nw-resize");
+    } else if (in_upper_right_corner(x, y, sebie) ||
+               (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION && sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER)) {
+        sebie_debug("UPPER_RIGHT\n");
+        gtk_widget_set_cursor_from_name(sebie->view.drawingarea1, "ne-resize");
+    } else if (in_lower_right_corner(x, y, sebie) ||
+               (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION && sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER)) {
+        sebie_debug("LOWER_RIGHT\n");
+        gtk_widget_set_cursor_from_name(sebie->view.drawingarea1, "se-resize");
+    } else if (in_lower_left_corner(x, y, sebie) ||
+               (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION && sebie->model.resize_corner == SEBIE_LOWER_LEFT_CORNER)) {
+        sebie_debug("LOWER_LEFT\n");
+        gtk_widget_set_cursor_from_name(sebie->view.drawingarea1, "sw-resize");
+    } else {
+        sebie_debug("DEFAULT\n");
+        gtk_widget_set_cursor_from_name(sebie->view.drawingarea1, "default");
+    }
+}
+
+
+void
+on_drawingarea1_drag_update(GtkGestureDrag *gesture,
+                            double xmove,
+                            double ymove,
+                            gpointer user_data)
+{
+    struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+    sebie_debug("on_drawingarea1_drag_update: xmove: %f ymove: %f\n", xmove, ymove);
+
+    gint x = sebie->model.selection_x + xmove + 0.5;
+    gint y = sebie->model.selection_y + ymove + 0.5;
+
+    if (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION) {
+        sebie_debug("resize selection\n");
+#if 0
+        gint x1 = sebie->model.selection_x;
+        gint y1 = sebie->model.selection_y;
+#endif
+        gint x2 = sebie->model.selection_x + sebie->model.selection_width;
+        gint y2 = sebie->model.selection_y + sebie->model.selection_height;
+
+        if (sebie->model.resize_corner == SEBIE_UPPER_LEFT_CORNER) {
+            gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
+            gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
+            double nfrac = (double)nwidth / (double)nheight;
+            if (nfrac >= sebie->model.selection_fac) {
+                sebie->model.selection_x = x2 - (double)nheight * sebie->model.selection_fac;
+                sebie->model.selection_y = y;
+                sebie->model.selection_width  = x2 - sebie->model.selection_x;
+                sebie->model.selection_height = y2 - sebie->model.selection_y;
+            } else {
+                sebie->model.selection_x = x;
+                sebie->model.selection_y = y2 - (double)nwidth / sebie->model.selection_fac;
+                sebie->model.selection_width = x2 - x;
+                sebie->model.selection_height = y2 - sebie->model.selection_y;
+            }
+        } else if (sebie->model.resize_corner == SEBIE_UPPER_RIGHT_CORNER) {
+            gint nwidth  = x - sebie->model.selection_x;
+            gint nheight = sebie->model.selection_y + sebie->model.selection_height - y;
+            double nfrac = (double)nwidth / (double)nheight;
+            if (nfrac >= sebie->model.selection_fac) {
+                sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+                sebie->model.selection_y = y;
+                sebie->model.selection_height = y2 - sebie->model.selection_y;
+            } else {
+                sebie->model.selection_width = nwidth;
+                sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+                sebie->model.selection_y = y2 - sebie->model.selection_height;
+            }
+        } else if (sebie->model.resize_corner == SEBIE_LOWER_RIGHT_CORNER) {
+            gint nwidth  = x - sebie->model.selection_x;
+            gint nheight = y - sebie->model.selection_y;
+            double nfrac = (double)nwidth / (double)nheight;
+            if (nfrac >= sebie->model.selection_fac) {
+                sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+                sebie->model.selection_height = nheight;
+            } else {
+                sebie->model.selection_width = nwidth;
+                sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+            }
+        } else { /* SEBIE_LOWER_LEFT_CORNER */
+            gint nwidth  = sebie->model.selection_x + sebie->model.selection_width - x;
+            gint nheight = y - sebie->model.selection_y;
+            double nfrac = (double)nwidth / (double)nheight;
+            if (nfrac >= sebie->model.selection_fac) {
+                sebie->model.selection_width = (double)nheight * sebie->model.selection_fac;
+                sebie->model.selection_height = nheight;
+                sebie->model.selection_x = x2 - sebie->model.selection_width;
+            } else {
+                sebie->model.selection_x = x;
+                sebie->model.selection_height = (double)nwidth / sebie->model.selection_fac;
+                sebie->model.selection_width = x2 - sebie->model.selection_x;
+            }
+        }
+        /* adjust if selection crossed the fix point */
+        if (sebie->model.selection_width < 0 &&
+            sebie->model.selection_height < 0) {
+            sebie->model.selection_x += sebie->model.selection_width;
+            sebie->model.selection_width = sebie->model.selection_width * (-1);
+            sebie->model.selection_y += sebie->model.selection_height;
+            sebie->model.selection_height = sebie->model.selection_height * (-1);
+            switch(sebie->model.resize_corner) {
+                case SEBIE_UPPER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_RIGHT_CORNER; break;
+                case SEBIE_UPPER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_LOWER_LEFT_CORNER; break;
+                case SEBIE_LOWER_RIGHT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_LEFT_CORNER; break;
+                case SEBIE_LOWER_LEFT_CORNER: sebie->model.resize_corner = SEBIE_UPPER_RIGHT_CORNER; break;
+            }
+        }
+    } else {
+        double xstart, ystart;
+        gtk_gesture_drag_get_start_point(gesture, &xstart, &ystart);
+        sebie->model.selection_x = xstart + xmove - sebie->model.mouse_pos_lx;
+        sebie->model.selection_y = ystart + ymove - sebie->model.mouse_pos_ly;
+    }
+
+    sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
+    sebie_update_view(sebie, NULL, NULL, 0, 0);
+
+    drawingarea1_update_cursor(x, y, sebie);
+}
+
+
+void
+on_drawingarea1_drag_end(GtkGestureDrag *gesture,
+                         double x,
+                         double y,
+                         gpointer user_data)
+{
+    struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+    sebie_debug("on_drawingarea1_drag_end: x: %f y: %f\n", x, y);
+
+    sebie->model.mouse_click_type = SEBIE_NOT_CLICKED;
+}
+
+
+void
+on_drawingarea1_motion(GtkEventControllerMotion *controller,
+                       double x,
+                       double y,
+                       gpointer user_data)
+{
+    struct sebie_s *sebie = (struct sebie_s *)user_data;
+
+    sebie_debug("on_drawingarea1_motion: x: %f y: %f\n", x, y);
+
+    drawingarea1_update_cursor(x, y, sebie);
+
+    sebie->dirty_tags |= SEBIE_UPDATE_SELECTION_RECT;
+    sebie_update_view(sebie, NULL, NULL, 0, 0);
+}
+#else
 gboolean
 on_drawingarea1_button_press_event (GtkWidget       *widget,
                     GdkEventButton  *event,
@@ -516,7 +880,10 @@ on_drawingarea1_button_press_event (GtkWidget       *widget,
 
   return FALSE;
 }
+#endif
 
+
+#if !GTK_CHECK_VERSION(4,0,0)
 gboolean
 on_drawingarea1_motion_notify_event (GtkWidget       *widget,
                      GdkEventMotion  *event,
@@ -550,8 +917,10 @@ on_drawingarea1_motion_notify_event (GtkWidget       *widget,
     sebie_debug("motion_notify_event %d %d \n", x, y);
     if (sebie->model.mouse_click_type == SEBIE_RESIZE_SELECTION) {
       sebie_debug("resize selection\n");
+#if 0
       gint x1 = sebie->model.selection_x;
       gint y1 = sebie->model.selection_y;
+#endif
       gint x2 = sebie->model.selection_x + sebie->model.selection_width;
       gint y2 = sebie->model.selection_y + sebie->model.selection_height;
 
@@ -684,3 +1053,16 @@ on_drawingarea1_motion_notify_event (GtkWidget       *widget,
 
   return TRUE;
 }
+#endif
+
+
+#if GTK_CHECK_VERSION(4,0,0)
+void
+on_error_dialog_response (GtkDialog *dialog,
+                          int response,
+                          gpointer user_data)
+{
+    sebie_debug("on_error_dialog_response() - response: %d\n", response);
+    gtk_window_destroy (GTK_WINDOW(dialog));
+}
+#endif
