@@ -21,7 +21,7 @@
 #ifndef _SEBIE_CALLBACKS_H_
 #define _SEBIE_CALLBACKS_H_  1
 
-#include "sebie.h"
+#include <gtk/gtk.h>
 
 void
 on_treeview_file_list_changed (GtkTreeSelection *selection,
@@ -47,10 +47,18 @@ void
 window1_destroy (GtkWidget *widget,
                  gpointer data);
 
+#if GTK_CHECK_VERSION(4,0,0)
+void
+drawingarea1_resize (GtkWidget *widget,
+                     gint width,
+                     gint height,
+                     gpointer data);
+#else
 void
 drawingarea1_size_allocate (GtkWidget *widget,
                             GtkAllocation *allocation,
                             gpointer data);
+#endif
 
 void
 on_button_quit_clicked (GtkButton *button,
@@ -84,7 +92,11 @@ void
 on_entry_selection_gamma_changed (GtkEditable *editable,
                                   gpointer user_data);
 
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+void
+on_button_grayscale_clicked(GtkToggleButton *button,
+                            gpointer user_data);
+#elif GTK_CHECK_VERSION(3,0,0)
 void
 on_button_grayscale_clicked(GtkToolItem *button,
                             gpointer user_data);
@@ -94,6 +106,38 @@ on_button_grayscale_clicked(GtkToggleButton *button,
                             gpointer user_data);
 #endif
 
+#if GTK_CHECK_VERSION(4,0,0)
+void
+drawingarea1_draw_function(GtkDrawingArea *area,
+                           cairo_t *cr,
+                           int width,
+                           int height,
+                           gpointer user_data);
+
+void
+on_drawingarea1_drag_begin(GtkGestureDrag *gesture,
+                           double x,
+                           double y,
+                           gpointer user_data);
+
+void
+on_drawingarea1_drag_update(GtkGestureDrag *gesture,
+                            double xmove,
+                            double ymove,
+                            gpointer user_data);
+
+void
+on_drawingarea1_drag_end(GtkGestureDrag *gesture,
+                         double xmove,
+                         double ymove,
+                         gpointer user_data);
+
+void
+on_drawingarea1_motion(GtkEventControllerMotion *controller,
+                       double x,
+                       double y,
+                       gpointer user_data);
+#else
 gboolean
 on_drawingarea1_expose_event (GtkWidget *widget,
                               GdkEventExpose *event,
@@ -108,5 +152,13 @@ gboolean
 on_drawingarea1_motion_notify_event (GtkWidget *widget,
                                      GdkEventMotion *event,
                                      gpointer user_data);
+#endif
+
+#if GTK_CHECK_VERSION(4,0,0)
+void
+on_error_dialog_response (GtkDialog *dialog,
+                          int response,
+                          gpointer user_data);
+#endif
 
 #endif /* _SEBIE_CALLBACKS_H_ */
