@@ -61,6 +61,11 @@ void
 sebie_regcomp(struct sebie_s *sebie)
 {
   int err;
+
+  if (sebie->model.regex_status) {
+      regfree(&sebie->model.regex);
+  }
+
   if ((err = regcomp(&sebie->model.regex, sebie->model.input_regex, 0)) != 0) {
     char buf[256];
     regerror(err, &sebie->model.regex, buf, 256);
