@@ -34,37 +34,59 @@
 GtkWidget *
 init_sebie_view(struct sebie_s *sebie)
 {
-  GtkWidget *tmp_toolbar_icon;
-  GtkAdjustment *gamma_adjustment;
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+#elif GTK_CHECK_VERSION(3,0,0)
   GtkToolItem *tmp_tool_item;
 #endif
 
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.window1 = gtk_window_new ();
+#else
   sebie->view.window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+#endif
   gchar * window1_title = "SeBIE " PACKAGE_VERSION ;
   gtk_widget_set_name (sebie->view.window1, window1_title);
   gtk_window_set_title (GTK_WINDOW (sebie->view.window1), window1_title);
 
 #if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.vbox1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+  sebie->view.grid1 = gtk_grid_new ();
+  gtk_grid_set_column_homogeneous (GTK_GRID(sebie->view.grid1), FALSE);
+  gtk_grid_set_row_homogeneous (GTK_GRID(sebie->view.grid1), FALSE);
+  gtk_widget_show (sebie->view.grid1);
 #else
   sebie->view.vbox1 = gtk_vbox_new (FALSE, 0);
-#endif
   gtk_widget_show (sebie->view.vbox1);
-  gtk_container_add (GTK_CONTAINER (sebie->view.window1), sebie->view.vbox1);
+#endif
 
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_window_set_child( GTK_WINDOW (sebie->view.window1), sebie->view.grid1);
+#else
+#if GTK_CHECK_VERSION(3,0,0)
+  gtk_container_add (GTK_CONTAINER (sebie->view.window1), sebie->view.grid1);
+#else
+  gtk_container_add (GTK_CONTAINER (sebie->view.window1), sebie->view.vbox1);
+#endif
+#endif
+
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.toolbar1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
   sebie->view.toolbar1 = gtk_toolbar_new ();
+#endif
   gtk_widget_show (sebie->view.toolbar1);
+#if GTK_CHECK_VERSION(3,0,0)
+  gtk_grid_attach (GTK_GRID (sebie->view.grid1), sebie->view.toolbar1, 0, 0, 2, 1);
+  gtk_widget_set_hexpand(sebie->view.toolbar1, TRUE);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox1), sebie->view.toolbar1, FALSE, FALSE, 0);
   gtk_toolbar_set_style (GTK_TOOLBAR (sebie->view.toolbar1), GTK_TOOLBAR_BOTH);
-
-#ifdef USE_ICON
-  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-quit", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
-#else
-  tmp_toolbar_icon = NULL;
 #endif
-#if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.button_quit =  gtk_tool_button_new(tmp_toolbar_icon, _("Quit"));
+
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.button_quit = gtk_button_new_with_label (_("Quit"));
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.button_quit);
+#elif GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_quit =  gtk_tool_button_new(NULL, _("Quit"));
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_quit, -1);
 #else
   sebie->view.button_quit = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
@@ -72,35 +94,16 @@ init_sebie_view(struct sebie_s *sebie)
                                                         NULL,
                                                         _("Quit"),
                                                         NULL, NULL,
-                                                        tmp_toolbar_icon, NULL, NULL);
+                                                        NULL, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
 #endif
   gtk_widget_show (GTK_WIDGET(sebie->view.button_quit));
 
-#ifdef USE_ICON
-  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-open", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
-#else
-  tmp_toolbar_icon = NULL;
-#endif
-
-#if 0
-  sebie->view.button_load = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
-                                GTK_TOOLBAR_CHILD_BUTTON,
-                                NULL,
-                                _("Load"),
-                                NULL, NULL,
-                                tmp_toolbar_icon, NULL, NULL);
-  gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
-  gtk_widget_show (sebie->view.button_load);
-#endif
-
-#ifdef USE_ICON
-  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-save", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
-#else
-  tmp_toolbar_icon = NULL;
-#endif
-#if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.button_save =  gtk_tool_button_new(tmp_toolbar_icon, _("Save"));
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.button_save = gtk_button_new_with_label (_("Save"));
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.button_save);
+#elif GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_save =  gtk_tool_button_new(NULL, _("Save"));
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_save, -1);
 #else
   sebie->view.button_save = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
@@ -108,18 +111,16 @@ init_sebie_view(struct sebie_s *sebie)
                                 NULL,
                                 _("Save"),
                                 NULL, NULL,
-                                tmp_toolbar_icon, NULL, NULL);
+                                NULL, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
 #endif
   gtk_widget_show (GTK_WIDGET(sebie->view.button_save));
 
-#ifdef USE_ICON
-  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-go-back", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
-#else
-  tmp_toolbar_icon = NULL;
-#endif
-#if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.button_rotate_left =  gtk_tool_button_new(tmp_toolbar_icon, _("<- Rotate "));
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.button_rotate_left = gtk_button_new_with_label (_("<- Rotate"));
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.button_rotate_left);
+#elif GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_rotate_left =  gtk_tool_button_new(NULL, _("<- Rotate"));
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_rotate_left, -1);
 #else
   sebie->view.button_rotate_left = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
@@ -127,18 +128,16 @@ init_sebie_view(struct sebie_s *sebie)
                                 NULL,
                                 _("<- Rotate"),
                                 NULL, NULL,
-                                tmp_toolbar_icon, NULL, NULL);
+                                NULL, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
 #endif
   gtk_widget_show (GTK_WIDGET(sebie->view.button_rotate_left));
 
-#ifdef USE_ICON
-  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-go-forward", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
-#else
-  tmp_toolbar_icon = NULL;
-#endif
-#if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.button_rotate_right =  gtk_tool_button_new(tmp_toolbar_icon, _("Rotate ->"));
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.button_rotate_right = gtk_button_new_with_label (_("Rotate ->"));
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.button_rotate_right);
+#elif GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_rotate_right =  gtk_tool_button_new(NULL, _("Rotate ->"));
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_rotate_right, -1);
 #else
   sebie->view.button_rotate_right = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
@@ -146,15 +145,16 @@ init_sebie_view(struct sebie_s *sebie)
                                 NULL,
                                 _("Rotate ->"),
                                 NULL, NULL,
-                                tmp_toolbar_icon, NULL, NULL);
+                                NULL, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
 #endif
   gtk_widget_show (GTK_WIDGET(sebie->view.button_rotate_right));
 
   sebie->view.label_selection_fac_width = gtk_label_new (_(" w:"));
   gtk_widget_show (sebie->view.label_selection_fac_width);
-
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.label_selection_fac_width);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_fac_width);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -166,7 +166,10 @@ init_sebie_view(struct sebie_s *sebie)
 
   sebie->view.entry_selection_fac_width = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_selection_fac_width);
-#if GTK_CHECK_VERSION(3,0,0)
+
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.entry_selection_fac_width);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.entry_selection_fac_width);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -178,7 +181,9 @@ init_sebie_view(struct sebie_s *sebie)
   gtk_entry_set_max_length (GTK_ENTRY (sebie->view.entry_selection_fac_width), 5);
   sebie->view.label_selection_fac_height = gtk_label_new (_(" h:"));
   gtk_widget_show (sebie->view.label_selection_fac_height);
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.label_selection_fac_height);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_fac_height);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -190,7 +195,9 @@ init_sebie_view(struct sebie_s *sebie)
 
   sebie->view.entry_selection_fac_height = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_selection_fac_height);
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.entry_selection_fac_height);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.entry_selection_fac_height);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -201,28 +208,28 @@ init_sebie_view(struct sebie_s *sebie)
   gtk_widget_set_size_request (sebie->view.entry_selection_fac_height, 50, 20);
   gtk_entry_set_max_length (GTK_ENTRY (sebie->view.entry_selection_fac_height), 5);
 
-#ifdef USE_TOOLBAR
-  tmp_toolbar_icon = gtk_image_new_from_stock ("gtk-refresh", gtk_toolbar_get_icon_size (GTK_TOOLBAR (sebie->view.toolbar1)));
-#else
-  tmp_toolbar_icon = NULL;
-#endif
-#if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.button_fac_swap =  gtk_tool_button_new(tmp_toolbar_icon, _("Swap"));
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.button_fac_swap = gtk_button_new_with_label (_("Swap"));
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.button_fac_swap);
+#elif GTK_CHECK_VERSION(3,0,0)
+  sebie->view.button_fac_swap =  gtk_tool_button_new(NULL, _("Swap"));
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_fac_swap, -1);
 #else
- sebie->view.button_fac_swap = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
+  sebie->view.button_fac_swap = gtk_toolbar_append_element (GTK_TOOLBAR (sebie->view.toolbar1),
                                 GTK_TOOLBAR_CHILD_BUTTON,
                                 NULL,
                                 _("Swap"),
                                 NULL, NULL,
-                                tmp_toolbar_icon, NULL, NULL);
+                                NULL, NULL, NULL);
   gtk_label_set_use_underline (GTK_LABEL (((GtkToolbarChild*) (g_list_last (GTK_TOOLBAR (sebie->view.toolbar1)->children)->data))->label), TRUE);
 #endif
   gtk_widget_show (GTK_WIDGET(sebie->view.button_fac_swap));
 
   sebie->view.label_selection_fac = gtk_label_new (_("label1"));
   gtk_widget_show (sebie->view.label_selection_fac);
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.label_selection_fac);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_fac);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -234,7 +241,9 @@ init_sebie_view(struct sebie_s *sebie)
 
   sebie->view.label_selection_gamma = gtk_label_new (_(" gamma:"));
   gtk_widget_show (sebie->view.label_selection_gamma);
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.label_selection_gamma);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.label_selection_gamma);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -244,17 +253,12 @@ init_sebie_view(struct sebie_s *sebie)
 #endif
   gtk_label_set_justify (GTK_LABEL (sebie->view.label_selection_gamma), GTK_JUSTIFY_LEFT);
 
-#if 1
-  gamma_adjustment = (GtkAdjustment *)gtk_adjustment_new(1.0, 0.1, 3.0, 0.1, 0.1, 0.0);
-  sebie->view.entry_selection_gamma = gtk_spin_button_new(gamma_adjustment,
-                                                          0.0,
-                                                          1);
-#else
-  sebie->view.entry_selection_gamma = gtk_entry_new ();
-#endif
+  sebie->view.entry_selection_gamma = gtk_spin_button_new_with_range(0.1, 3.0, 0.1);
   gtk_widget_show (sebie->view.entry_selection_gamma);
 
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.entry_selection_gamma);
+#elif GTK_CHECK_VERSION(3,0,0)
   tmp_tool_item = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(tmp_tool_item), sebie->view.entry_selection_gamma);
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), tmp_tool_item, -1);
@@ -263,9 +267,15 @@ init_sebie_view(struct sebie_s *sebie)
   gtk_toolbar_append_widget (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.entry_selection_gamma, NULL, NULL);
 #endif
   gtk_widget_set_size_request (sebie->view.entry_selection_gamma, 50, 20);
+#if GTK_CHECK_VERSION(4,0,0)
+#else
   gtk_entry_set_max_length (GTK_ENTRY (sebie->view.entry_selection_gamma), 5);
+#endif
 
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.button_grayscale = gtk_toggle_button_new_with_label (_("Grayscale"));
+  gtk_box_append( GTK_BOX(sebie->view.toolbar1), sebie->view.button_grayscale);
+#elif GTK_CHECK_VERSION(3,0,0)
   sebie->view.button_grayscale = gtk_toggle_tool_button_new();
   gtk_tool_button_set_label(GTK_TOOL_BUTTON(sebie->view.button_grayscale), _("Grayscale"));
   gtk_toolbar_insert (GTK_TOOLBAR (sebie->view.toolbar1), sebie->view.button_grayscale, -1);
@@ -280,13 +290,11 @@ init_sebie_view(struct sebie_s *sebie)
 #endif
   gtk_widget_show (GTK_WIDGET(sebie->view.button_grayscale));
 
-#if GTK_CHECK_VERSION(3,0,0)
-  sebie->view.hpaned1 = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
-#else
+#if !GTK_CHECK_VERSION(3,0,0)
   sebie->view.hpaned1 = gtk_hpaned_new ();
-#endif
   gtk_widget_show (sebie->view.hpaned1);
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox1), sebie->view.hpaned1, TRUE, TRUE, 0);
+#endif
 
 #if GTK_CHECK_VERSION(3,0,0)
   sebie->view.vbox2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
@@ -294,49 +302,101 @@ init_sebie_view(struct sebie_s *sebie)
   sebie->view.vbox2 = gtk_vbox_new (FALSE, 0);
 #endif
   gtk_widget_show (sebie->view.vbox2);
+#if GTK_CHECK_VERSION(3,0,0)
+  gtk_grid_attach (GTK_GRID (sebie->view.grid1), sebie->view.vbox2, 0, 1, 1, 1);
+  gtk_widget_set_size_request (sebie->view.vbox2, 150, -1);
+  gtk_widget_set_hexpand(sebie->view.vbox2, FALSE);
+  gtk_widget_set_vexpand(sebie->view.vbox2, TRUE);
+#else
   gtk_paned_pack1 (GTK_PANED (sebie->view.hpaned1), sebie->view.vbox2, FALSE, TRUE);
-
+#endif
   sebie->view.label_input_path = gtk_label_new ("Path:");
   gtk_widget_show (sebie->view.label_input_path);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.label_input_path);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.label_input_path, FALSE, FALSE, 0);
+#endif
 
   sebie->view.entry_input_path = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_input_path);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.entry_input_path);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.entry_input_path, FALSE, FALSE, 0);
+#endif
 
   sebie->view.label_input_regex = gtk_label_new ("Regex:");
   gtk_widget_show (sebie->view.label_input_regex);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.label_input_regex);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.label_input_regex, FALSE, FALSE, 0);
+#endif
 
   sebie->view.entry_input_regex = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_input_regex);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.entry_input_regex);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.entry_input_regex, FALSE, FALSE, 0);
+#endif
 
   sebie->view.label_output_format = gtk_label_new ("Output Format:");
   gtk_widget_show (sebie->view.label_output_format);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.label_output_format);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.label_output_format, FALSE, FALSE, 0);
+#endif
 
   sebie->view.entry_output_format = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_output_format);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.entry_output_format);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.entry_output_format, FALSE, FALSE, 0);
+#endif
 
   sebie->view.label_output_file = gtk_label_new ("Output:");
   gtk_widget_show (sebie->view.label_output_file);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.label_output_file);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.label_output_file, FALSE, FALSE, 0);
+#endif
 
   sebie->view.entry_output_file = gtk_entry_new ();
   gtk_widget_show (sebie->view.entry_output_file);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.entry_output_file);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.entry_output_file, FALSE, FALSE, 0);
+#endif
 
+#if GTK_CHECK_VERSION(4,0,0)
+  sebie->view.scrolledwindow1 = gtk_scrolled_window_new ();
+  gtk_widget_set_vexpand(sebie->view.scrolledwindow1, TRUE);
+
+#else
   sebie->view.scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
+#endif
   gtk_widget_show (sebie->view.scrolledwindow1);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_box_append( GTK_BOX(sebie->view.vbox2), sebie->view.scrolledwindow1);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox2), sebie->view.scrolledwindow1, TRUE, TRUE, 0);
+#endif
 
   sebie->view.treeview_file_list = gtk_tree_view_new ();
   gtk_tree_view_set_model(GTK_TREE_VIEW(sebie->view.treeview_file_list), GTK_TREE_MODEL(sebie->model.file_list));
 
   gtk_widget_show (sebie->view.treeview_file_list);
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_scrolled_window_set_child( GTK_SCROLLED_WINDOW (sebie->view.scrolledwindow1), sebie->view.treeview_file_list);
+#else
   gtk_container_add (GTK_CONTAINER (sebie->view.scrolledwindow1), sebie->view.treeview_file_list);
+#endif
   GtkCellRenderer *renderer;
   GtkTreeViewColumn *column;
   renderer = gtk_cell_renderer_text_new();
@@ -345,7 +405,14 @@ init_sebie_view(struct sebie_s *sebie)
 
   sebie->view.drawingarea1 = gtk_drawing_area_new ();
   gtk_widget_show (sebie->view.drawingarea1);
+#if GTK_CHECK_VERSION(3,0,0)
+  gtk_grid_attach (GTK_GRID (sebie->view.grid1), sebie->view.drawingarea1, 1, 1, 1, 1);
+  gtk_widget_set_hexpand(sebie->view.drawingarea1, TRUE);
+  gtk_widget_set_vexpand(sebie->view.drawingarea1, TRUE);
+  gtk_widget_set_size_request (sebie->view.drawingarea1, sebie->model.window1_width - 150, -1 /*sebie->model.window1_height - 50*/);
+#else
   gtk_paned_pack2 (GTK_PANED (sebie->view.hpaned1), sebie->view.drawingarea1, TRUE, TRUE);
+#endif
 
   gtk_window_set_default_size(GTK_WINDOW (sebie->view.window1),
                               sebie->model.window1_width,
@@ -354,7 +421,12 @@ init_sebie_view(struct sebie_s *sebie)
   sebie->view.statusbar1 = gtk_statusbar_new ();
   gtk_widget_show (sebie->view.statusbar1);
   sebie->view.statusbar1_context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(sebie->view.statusbar1), "SomeContext");
+#if GTK_CHECK_VERSION(3,0,0)
+  gtk_grid_attach (GTK_GRID (sebie->view.grid1), sebie->view.statusbar1, 0, 2, 2, 1);
+  gtk_widget_set_hexpand(sebie->view.statusbar1, TRUE);
+#else
   gtk_box_pack_start (GTK_BOX (sebie->view.vbox1), sebie->view.statusbar1, FALSE, FALSE, 0);
+#endif
 
   GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(sebie->view.treeview_file_list));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
@@ -371,23 +443,27 @@ init_sebie_view(struct sebie_s *sebie)
                     G_CALLBACK (on_entry_output_format_changed),
                     sebie);
 
+#if GTK_CHECK_VERSION(4,0,0)
+  g_signal_connect (G_OBJECT (sebie->view.window1), "close-request",
+                    G_CALLBACK (window1_delete_event), NULL);
+#else
   g_signal_connect (G_OBJECT (sebie->view.window1), "delete-event",
                     G_CALLBACK (window1_delete_event), NULL);
+#endif
   g_signal_connect (G_OBJECT (sebie->view.window1), "destroy",
                     G_CALLBACK (window1_destroy), NULL);
 
+#if GTK_CHECK_VERSION(4,0,0)
+  g_signal_connect (G_OBJECT (sebie->view.drawingarea1), "resize",
+                    G_CALLBACK (drawingarea1_resize), sebie);
+#else
   g_signal_connect (G_OBJECT (sebie->view.drawingarea1), "size-allocate",
                     G_CALLBACK (drawingarea1_size_allocate), sebie);
+#endif
 
   g_signal_connect ((gpointer) sebie->view.button_quit, "clicked",
                     G_CALLBACK (on_button_quit_clicked),
                     sebie);
-#if 0
-  g_signal_connect ((gpointer) sebie->view.button_load, "clicked",
-                    G_CALLBACK (on_button_load_clicked),
-                    sebie);
-#endif
-
   g_signal_connect ((gpointer) sebie->view.button_save, "clicked",
                     G_CALLBACK (on_button_save_clicked),
                     sebie);
@@ -400,7 +476,9 @@ init_sebie_view(struct sebie_s *sebie)
   g_signal_connect ((gpointer) sebie->view.button_fac_swap, "clicked",
                     G_CALLBACK (on_button_fac_swap),
                     sebie);
-#if GTK_CHECK_VERSION(3,0,0)
+#if GTK_CHECK_VERSION(4,0,0)
+  gtk_drawing_area_set_draw_func (GTK_DRAWING_AREA (sebie->view.drawingarea1), drawingarea1_draw_function, sebie, NULL);
+#elif GTK_CHECK_VERSION(3,0,0)
   g_signal_connect ((gpointer) sebie->view.drawingarea1, "draw",
                     G_CALLBACK (on_drawingarea1_expose_event),
                     sebie);
@@ -409,12 +487,25 @@ init_sebie_view(struct sebie_s *sebie)
                     G_CALLBACK (on_drawingarea1_expose_event),
                     sebie);
 #endif
+#if GTK_CHECK_VERSION(4,0,0)
+  GtkGesture *drag = gtk_gesture_drag_new ();
+  gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (drag), GDK_BUTTON_PRIMARY);
+  gtk_widget_add_controller (sebie->view.drawingarea1, GTK_EVENT_CONTROLLER (drag));
+  g_signal_connect (drag, "drag-begin", G_CALLBACK (on_drawingarea1_drag_begin), sebie);
+  g_signal_connect (drag, "drag-update", G_CALLBACK (on_drawingarea1_drag_update), sebie);
+  g_signal_connect (drag, "drag-end", G_CALLBACK (on_drawingarea1_drag_end), sebie);
+
+  GtkEventController* motion_controller = gtk_event_controller_motion_new();
+  g_signal_connect (motion_controller, "motion", G_CALLBACK (on_drawingarea1_motion), sebie);
+  gtk_widget_add_controller (sebie->view.drawingarea1, motion_controller);
+#else
   g_signal_connect ((gpointer) sebie->view.drawingarea1, "button_press_event",
                     G_CALLBACK (on_drawingarea1_button_press_event),
                     sebie);
   g_signal_connect ((gpointer) sebie->view.drawingarea1, "motion_notify_event",
                     G_CALLBACK (on_drawingarea1_motion_notify_event),
                     sebie);
+#endif
   g_signal_connect ((gpointer) sebie->view.entry_selection_fac_width, "changed",
                     G_CALLBACK (on_entry_selection_fac_width_changed),
                     sebie);
@@ -428,20 +519,23 @@ init_sebie_view(struct sebie_s *sebie)
                     G_CALLBACK (on_button_grayscale_clicked),
                     sebie);
 
+#if !GTK_CHECK_VERSION(4,0,0)
   gtk_widget_set_events (sebie->view.drawingarea1, GDK_EXPOSURE_MASK
                          | GDK_LEAVE_NOTIFY_MASK
                          | GDK_BUTTON_PRESS_MASK
                          | GDK_POINTER_MOTION_MASK
                          | GDK_POINTER_MOTION_HINT_MASK);
+#endif
 
   /* Store pointers to all widgets, for use by lookup_widget(). */
   GLADE_HOOKUP_OBJECT_NO_REF (sebie->view.window1, sebie->view.window1, "window1");
+#if GTK_CHECK_VERSION(3,0,0)
+  GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.grid1, "grid1");
+#else
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.vbox1, "vbox1");
+#endif
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.toolbar1, "toolbar1");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.button_quit, "button_quit");
-#if 0
-  GLADE_HOOKUP_OBJECT (sebie->view.window1,sebie->view. button_load, "button_load");
-#endif
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.button_save, "button_save");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.button_rotate_left, "button_rotate_left");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.button_rotate_right, "button_rotate_right");
@@ -449,10 +543,9 @@ init_sebie_view(struct sebie_s *sebie)
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.entry_selection_fac_height, "entry_selection_fac_height");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.button_fac_swap, "button_frac_swap");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.label_selection_fac, "label_selection_fac");
-#if 0
-  GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.button7, "button7");
-#endif
+#if !GTK_CHECK_VERSION(3,0,0)
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.hpaned1, "hpaned1");
+#endif
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.vbox2, "vbox2");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.entry_input_path, "entry_input_path");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.entry_output_file, "entry_output_file");
@@ -461,7 +554,5 @@ init_sebie_view(struct sebie_s *sebie)
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.drawingarea1, "drawingarea1");
   GLADE_HOOKUP_OBJECT (sebie->view.window1, sebie->view.statusbar1, "statusbar1");
 
-  //gtk_widget_grab_default (sebie->view.drawingarea1);
-
   return sebie->view.window1;
 }
